@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -36,8 +36,7 @@
 namespace love
 {
 
-static thread::Mutex *gcmutex = 0;
-void *_gcmutex = 0;
+static thread::Mutex *gcmutex = nullptr;
 
 /**
  * Called when an object is collected. The object is released
@@ -46,25 +45,15 @@ void *_gcmutex = 0;
 static int w__gc(lua_State *L)
 {
 	if (!gcmutex)
-	{
 		gcmutex = thread::newMutex();
-		_gcmutex = (void *) gcmutex;
-	}
 
-	Proxy *p = (Proxy *)lua_touserdata(L, 1);
-	Object *t = (Object *)p->data;
+	Proxy *p = (Proxy *) lua_touserdata(L, 1);
+	Object *object = (Object *) p->data;
 
 	thread::Lock lock(gcmutex);
 
-	int numretains = p->retains;
-	if (numretains >= 0)
-		numretains = std::min(numretains, t->getReferenceCount());
+	object->release();
 
-	for (int i = numretains; i > 0; i--)
-		t->release();
-
-	// Signal that this Proxy is dead.
-	p->retains = -1;
 	return 0;
 }
 
@@ -228,7 +217,6 @@ int luax_register_module(lua_State *L, const WrappedModule &m)
 	luax_insistregistry(L, REGISTRY_MODULES);
 
 	Proxy *p = (Proxy *)lua_newuserdata(L, sizeof(Proxy));
-	p->retains = 1;
 	p->data = m.module;
 	p->flags = m.flags;
 
@@ -394,20 +382,27 @@ int luax_register_searcher(lua_State *L, lua_CFunction f, int pos)
 	return 0;
 }
 
-void luax_rawnewtype(lua_State *L, const char *name, bits flags, love::Object *data, bool own)
+void luax_rawnewtype(lua_State *L, const char *name, bits flags, love::Object *object)
 {
 	Proxy *u = (Proxy *)lua_newuserdata(L, sizeof(Proxy));
 
-	u->data = (void *) data;
+	object->retain();
+
+	u->data = (void *) object;
 	u->flags = flags;
-	u->retains = own ? 1 : 0;
 
 	luaL_newmetatable(L, name);
 	lua_setmetatable(L, -2);
 }
 
-void luax_pushtype(lua_State *L, const char *name, bits flags, love::Object *data, bool own)
+void luax_pushtype(lua_State *L, const char *name, bits flags, love::Object *object)
 {
+	if (object == nullptr)
+	{
+		lua_pushnil(L);
+		return;
+	}
+
 	// Fetch the registry table of instantiated types.
 	luax_getregistry(L, REGISTRY_TYPES);
 
@@ -415,11 +410,11 @@ void luax_pushtype(lua_State *L, const char *name, bits flags, love::Object *dat
 	if (!lua_istable(L, -1))
 	{
 		lua_pop(L, 1);
-		return luax_rawnewtype(L, name, flags, data, own);
+		return luax_rawnewtype(L, name, flags, object);
 	}
 
-	// Get the value of lovetypes[data] on the stack.
-	lua_pushlightuserdata(L, (void *) data);
+	// Get the value of lovetypes[object] on the stack.
+	lua_pushlightuserdata(L, (void *) object);
 	lua_gettable(L, -2);
 
 	// If the Proxy userdata isn't in the instantiated types table yet, add it.
@@ -427,38 +422,18 @@ void luax_pushtype(lua_State *L, const char *name, bits flags, love::Object *dat
 	{
 		lua_pop(L, 1);
 
-		luax_rawnewtype(L, name, flags, data, own);
+		luax_rawnewtype(L, name, flags, object);
 
-		lua_pushlightuserdata(L, (void *) data);
+		lua_pushlightuserdata(L, (void *) object);
 		lua_pushvalue(L, -2);
 
-		// lovetypes[data] = Proxy.
+		// lovetypes[object] = Proxy.
 		lua_settable(L, -4);
-
-		// Remove the lovetypes table from the stack.
-		lua_remove(L, -2);
-
-		// The Proxy userdata remains at the top of the stack.
-		return;
 	}
 
 	// Remove the lovetypes table from the stack.
 	lua_remove(L, -2);
 
-	// If the object should be released on GC and we already have a stored
-	// Proxy, we should tell the Proxy that the object was retained again.
-	if (own)
-	{
-		Proxy *p = (Proxy *) lua_touserdata(L, -1);
-
-		thread::EmptyLock lock;
-		if (gcmutex)
-			lock.setLock(gcmutex);
-
-		if (p->retains >= 0)
-			++(p->retains);
-	}
-
 	// Keep the Proxy userdata on the stack.
 }
 
@@ -486,6 +461,10 @@ int luax_getfunction(lua_State *L, const char *mod, const char *fn)
 
 int luax_convobj(lua_State *L, int idx, const char *mod, const char *fn)
 {
+	// Convert to absolute index if necessary.
+	if (idx < 0 && idx > LUA_REGISTRYINDEX)
+		idx += lua_gettop(L) + 1;
+
 	// Convert string to a file.
 	luax_getfunction(L, mod, fn);
 	lua_pushvalue(L, idx); // The initial argument.
@@ -538,7 +517,7 @@ int luax_insist(lua_State *L, int idx, const char *k)
 {
 	// Convert to absolute index if necessary.
 	if (idx < 0 && idx > LUA_REGISTRYINDEX)
-		idx = lua_gettop(L) + ++idx;
+		idx += lua_gettop(L) + 1;
 
 	lua_getfield(L, idx, k);
 
