@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -37,6 +37,7 @@ Audio::PoolThread::PoolThread(Pool *pool)
 	, finish(false)
 {
 	mutex = thread::newMutex();
+	threadName = "AudioPool";
 }
 
 Audio::PoolThread::~PoolThread()
@@ -255,6 +256,17 @@ void Audio::setVelocity(float *v)
 	alListenerfv(AL_VELOCITY, v);
 }
 
+void Audio::setDopplerScale(float scale)
+{
+	if (scale >= 0.0f)
+		alDopplerFactor(scale);
+}
+
+float Audio::getDopplerScale() const
+{
+	return alGetFloat(AL_DOPPLER_FACTOR);
+}
+
 void Audio::record()
 {
 	if (!canRecord()) return;
