@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -39,13 +39,11 @@ ImageRasterizer::ImageRasterizer(love::image::ImageData *data, uint32 *glyphs, i
 	, glyphs(glyphs)
 	, numglyphs(numglyphs)
 {
-	imageData->retain();
 	load();
 }
 
 ImageRasterizer::~ImageRasterizer()
 {
-	imageData->release();
 }
 
 int ImageRasterizer::getLineHeight() const
