@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -25,6 +25,8 @@
 #include "font/wrap_GlyphData.h"
 #include "font/wrap_Rasterizer.h"
 
+#include "filesystem/wrap_Filesystem.h"
+
 #include "TrueTypeRasterizer.h"
 
 namespace love
@@ -34,55 +36,59 @@ namespace font
 namespace freetype
 {
 
-static Font *instance = 0;
+#define instance() (Module::getInstance<Font>(Module::M_FONT))
 
 int w_newRasterizer(lua_State *L)
 {
-	// Convert to FileData, if necessary.
-	if (lua_isstring(L, 1) || luax_istype(L, 1, FILESYSTEM_FILE_T))
-		luax_convobj(L, 1, "filesystem", "newFileData");
+	Rasterizer *t = nullptr;
 
-	Rasterizer *t = 0;
-
-	EXCEPT_GUARD(
-		if (luax_istype(L, 1, IMAGE_IMAGE_DATA_T))
-		{
-			love::image::ImageData *d = luax_checktype<love::image::ImageData>(L, 1, "ImageData", IMAGE_IMAGE_DATA_T);
-			const char *g = luaL_checkstring(L, 2);
-			std::string glyphs(g);
-			t = instance->newRasterizer(d, glyphs);
-		}
-		else if (luax_istype(L, 1, DATA_T))
-		{
-			Data *d = luax_checkdata(L, 1);
-			int size = luaL_checkint(L, 2);
-			t = instance->newRasterizer(d, size);
-		}
-	)
+	if (luax_istype(L, 1, IMAGE_IMAGE_DATA_T))
+	{
+		love::image::ImageData *d = luax_checktype<love::image::ImageData>(L, 1, "ImageData", IMAGE_IMAGE_DATA_T);
+		const char *g = luaL_checkstring(L, 2);
+		std::string glyphs(g);
+		luax_catchexcept(L, [&](){ t = instance()->newRasterizer(d, glyphs); });
+	}
+	else if (lua_type(L, 1) == LUA_TSTRING || luax_istype(L, 1, FILESYSTEM_FILE_T) || luax_istype(L, 1, FILESYSTEM_FILE_DATA_T))
+	{
+		love::filesystem::FileData *d = love::filesystem::luax_getfiledata(L, 1);
+		int size = luaL_optint(L, 2, 12);
+		luax_catchexcept(L,
+			[&]() { t = instance()->newRasterizer(d, size); },
+			[&]() { d->release(); }
+		);
+	}
+	else
+	{
+		// Default font (Vera.)
+		int size = luaL_optint(L, 1, 12);
+		luax_catchexcept(L, [&]() { t = instance()->newRasterizer(size); });
+	}
 
 	luax_pushtype(L, "Rasterizer", FONT_RASTERIZER_T, t);
+	t->release();
 	return 1;
 }
 
 int w_newGlyphData(lua_State *L)
 {
 	Rasterizer *r = luax_checkrasterizer(L, 1);
-	GlyphData *t = 0;
+	GlyphData *t = nullptr;
 
 	// newGlyphData accepts a unicode character or a codepoint number.
 	if (lua_type(L, 2) == LUA_TSTRING)
 	{
 		std::string glyph = luax_checkstring(L, 2);
-
-		EXCEPT_GUARD(t = instance->newGlyphData(r, glyph);)
+		luax_catchexcept(L, [&](){ t = instance()->newGlyphData(r, glyph); });
 	}
 	else
 	{
 		uint32 g = (uint32) luaL_checknumber(L, 2);
-		t = instance->newGlyphData(r, g);
+		t = instance()->newGlyphData(r, g);
 	}
 
 	luax_pushtype(L, "GlyphData", FONT_GLYPH_DATA_T, t);
+	t->release();
 	return 1;
 }
 
@@ -103,9 +109,10 @@ static const lua_CFunction types[] =
 
 extern "C" int luaopen_love_font(lua_State *L)
 {
-	if (instance == 0)
+	Font *instance = instance();
+	if (instance == nullptr)
 	{
-		EXCEPT_GUARD(instance = new Font();)
+		luax_catchexcept(L, [&](){ instance = new Font(); });
 	}
 	else
 		instance->retain();
