@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -26,9 +26,11 @@
 // LOVE
 #include "graphics/Color.h"
 #include "graphics/Texture.h"
+#include "common/Matrix.h"
 
 // C++
 #include <vector>
+#include <stack>
 
 // The last argument to AttribPointer takes a buffer offset casted to a pointer.
 #define BUFFER_OFFSET(i) ((char *) NULL + (i))
@@ -101,6 +103,42 @@ public:
 		GLenum func;
 	};
 
+	struct
+	{
+		std::vector<Matrix> transform;
+		std::vector<Matrix> projection;
+	} matrices;
+
+	class TempTransform
+	{
+	public:
+
+		TempTransform(OpenGL &gl)
+			: gl(gl)
+		{
+			gl.pushTransform();
+		}
+
+		~TempTransform()
+		{
+			gl.popTransform();
+		}
+
+		Matrix &get()
+		{
+			return gl.getTransform();
+		}
+
+	private:
+		OpenGL &gl;
+	};
+
+	struct Stats
+	{
+		size_t textureMemory;
+		int    drawCalls;
+	} stats;
+
 	OpenGL();
 
 	/**
@@ -116,6 +154,10 @@ public:
 	 **/
 	void deInitContext();
 
+	void pushTransform();
+	void popTransform();
+	Matrix &getTransform();
+
 	/**
 	 * Set up necessary state (LOVE-provided shader uniforms, etc.) for drawing.
 	 * This *MUST* be called directly before OpenGL drawing functions.
@@ -123,14 +165,11 @@ public:
 	void prepareDraw();
 
 	/**
-	 * glDrawArraysInstanced with a pseudo-instancing fallback.
+	 * glDrawArrays and glDrawElements which increment the draw-call counter by
+	 * themselves.
 	 **/
-	void drawArraysInstanced(GLenum mode, GLint first, GLsizei count, GLsizei primcount);
-
-	/**
-	 * glDrawElementsInstanced with a pseudo-instancing fallback.
-	 **/
-	void drawElementsInstanced(GLenum mode, GLsizei count, GLenum type, const void *indices, GLsizei primcount);
+	void drawArrays(GLenum mode, GLint first, GLsizei count);
+	void drawElements(GLenum mode, GLsizei count, GLenum type, const void *indices);
 
 	/**
 	 * Sets the current constant color.
@@ -218,21 +257,11 @@ public:
 	 **/
 	float setTextureFilter(graphics::Texture::Filter &f);
 
-	/**
-	 * Returns the texture filter mode for the currently bound texture.
-	 **/
-	graphics::Texture::Filter getTextureFilter();
-
 	/**
 	 * Sets the texture wrap mode for the currently bound texture.
 	 **/
 	void setTextureWrap(const graphics::Texture::Wrap &w);
 
-	/**
-	 * Returns the texture wrap mode for the currently bound texture.
-	 **/
-	graphics::Texture::Wrap getTextureWrap();
-
 	/**
 	 * Returns the maximum supported width or height of a texture.
 	 **/
@@ -243,6 +272,8 @@ public:
 	 **/
 	int getMaxRenderTargets() const;
 
+	void updateTextureMemorySize(size_t oldsize, size_t newsize);
+
 	/**
 	 * Get the GPU vendor of this OpenGL context.
 	 **/
@@ -258,6 +289,7 @@ private:
 	void initVendor();
 	void initOpenGLFunctions();
 	void initMaxValues();
+	void initMatrices();
 	void createDefaultTexture();
 
 	bool contextInitialized;
@@ -287,8 +319,8 @@ private:
 
 		BlendState blend;
 
-		// The last ID value used for pseudo-instancing.
-		int lastPseudoInstanceID;
+		Matrix lastProjectionMatrix;
+		Matrix lastTransformMatrix;
 
 	} state;
 
