@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -27,6 +27,7 @@
 #include "common/Vector.h"
 #include "graphics/Drawable.h"
 #include "graphics/Color.h"
+#include "graphics/Quad.h"
 #include "Texture.h"
 
 // STL
@@ -161,7 +162,7 @@ public:
 	 * @param[out] min The minimum life.
 	 * @param[out] max The maximum life.
 	 **/
-	void getParticleLifetime(float *min, float *max) const;
+	void getParticleLifetime(float &min, float &max) const;
 
 	/**
 	 * Sets the position of the center of the emitter.
@@ -248,7 +249,7 @@ public:
 	 * @param[out] min The minimum speed.
 	 * @param[out] max The maximum speed.
 	 **/
-	void getSpeed(float *min, float *max) const;
+	void getSpeed(float &min, float &max) const;
 
 	/**
 	 * Sets the linear acceleration (the acceleration along the x and y axes).
@@ -271,7 +272,7 @@ public:
 	 * @param[out] min The minimum acceleration.
 	 * @param[out] max The maximum acceleration.
 	 **/
-	void getLinearAcceleration(love::Vector *min, love::Vector *max) const;
+	void getLinearAcceleration(love::Vector &min, love::Vector &max) const;
 
 	/**
 	 * Sets the radial acceleration (the acceleration towards the particle emitter).
@@ -291,7 +292,7 @@ public:
 	 * @param[out] min The minimum amount of radial acceleration.
 	 * @param[out] max The maximum amount of radial acceleration.
 	 **/
-	void getRadialAcceleration(float *min, float *max) const;
+	void getRadialAcceleration(float &min, float &max) const;
 
 	/**
 	 * Sets the tangential acceleration (the acceleration perpendicular to the particle's direction).
@@ -311,7 +312,18 @@ public:
 	 * @param[out] min The minimum tangential acceleration.
 	 * @param[out] max The maximum tangential acceleration.
 	 **/
-	void getTangentialAcceleration(float *min, float *max) const;
+	void getTangentialAcceleration(float &min, float &max) const;
+
+	/**
+	 * Sets the amount of linear damping. Damping reduces the velocity of
+	 * particles over time. A value of 0 corresponds to no damping.
+	 **/
+	void setLinearDamping(float min, float max);
+
+	/**
+	 * Gets the current amount of linear damping.
+	 **/
+	void getLinearDamping(float &min, float &max) const;
 
 	/**
 	 * Sets the size of the sprite (1.0 being the default size).
@@ -359,7 +371,7 @@ public:
 	 * @param[out] min The minimum initial rotation.
 	 * @param[out] max The maximum initial rotation.
 	 **/
-	void getRotation(float *min, float *max) const;
+	void getRotation(float &min, float &max) const;
 
 	/**
 	 * Sets the spin of the sprite.
@@ -379,7 +391,7 @@ public:
 	 * @param[out] start The initial spin, in radians / s.
 	 * @param[out] end The final spin, in radians / s.
 	 **/
-	void getSpin(float *start, float *end) const;
+	void getSpin(float &start, float &end) const;
 
 	/**
 	 * Sets the variation of the start spin (0 being no variation and 1 being a random spin between start and end).
@@ -421,6 +433,17 @@ public:
 	 **/
 	std::vector<Color> getColor() const;
 
+	/**
+	 * Sets a list of Quads to use for particles over their lifetime.
+	 **/
+	void setQuads(const std::vector<Quad *> &newQuads);
+	void setQuads();
+
+	/**
+	 * Gets the Quads used when drawing the particles.
+	 **/
+	std::vector<Quad *> getQuads() const;
+
 	/**
 	 * sets whether particle angles & rotations are relative to their velocities.
 	 **/
@@ -516,11 +539,13 @@ protected:
 		// Particles gravitate towards this point.
 		love::Vector origin;
 
-		love::Vector speed;
+		love::Vector velocity;
 		love::Vector linearAcceleration;
 		float radialAcceleration;
 		float tangentialAcceleration;
 
+		float linearDamping;
+
 		float size;
 		float sizeOffset;
 		float sizeIntervalSize;
@@ -531,6 +556,8 @@ protected:
 		float spinEnd;
 
 		Colorf color;
+
+		int quadIndex;
 	};
 
 	// Pointer to the beginning of the allocated memory.
@@ -549,7 +576,7 @@ protected:
 	Vertex *particleVerts;
 
 	// The texture to be drawn.
-	Texture *texture;
+	StrongRef<Texture> texture;
 
 	// Whether the particle emitter is active.
 	bool active;
@@ -605,6 +632,9 @@ protected:
 	float tangentialAccelerationMin;
 	float tangentialAccelerationMax;
 
+	float linearDampingMin;
+	float linearDampingMax;
+
 	// Size.
 	std::vector<float> sizes;
 	float sizeVariation;
@@ -625,6 +655,9 @@ protected:
 	// Color.
 	std::vector<Colorf> colors;
 
+	// Quads.
+	std::vector<StrongRef<Quad>> quads;
+
 	bool relativeRotation;
 
 	void createBuffers(size_t size);
