@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -64,6 +64,7 @@ int w_setColor(lua_State *L);
 int w_getColor(lua_State *L);
 int w_setBackgroundColor(lua_State *L);
 int w_getBackgroundColor(lua_State *L);
+int w_setNewFont(lua_State *L);
 int w_setFont(lua_State *L);
 int w_getFont(lua_State *L);
 int w_setColorMask(lua_State *L);
@@ -93,7 +94,10 @@ int w_getCanvas(lua_State *L);
 int w_setShader(lua_State *L);
 int w_getShader(lua_State *L);
 int w_isSupported(lua_State *L);
+int w_getCanvasFormats(lua_State *L);
+int w_getCompressedImageFormats(lua_State *L);
 int w_getRendererInfo(lua_State *L);
+int w_getStats(lua_State *L);
 int w_getSystemLimit(lua_State *L);
 int w_draw(lua_State *L);
 int w_print(lua_State *L);
