@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -45,6 +45,7 @@ int w_BezierCurve_getDerivative(lua_State *L)
 	BezierCurve *curve = luax_checkbeziercurve(L, 1);
 	BezierCurve *deriv = new BezierCurve(curve->getDerivative());
 	luax_pushtype(L, "BezierCurve", MATH_BEZIER_CURVE_T, deriv);
+	deriv->release();
 	return 1;
 }
 
@@ -56,11 +57,11 @@ int w_BezierCurve_getControlPoint(lua_State *L)
 	if (idx > 0) // 1-indexing
 		idx--;
 
-	EXCEPT_GUARD(
+	luax_catchexcept(L, [&]() {
 		Vector v = curve->getControlPoint(idx);
 		lua_pushnumber(L, v.x);
 		lua_pushnumber(L, v.y);
-	)
+	});
 
 	return 2;
 }
@@ -75,7 +76,7 @@ int w_BezierCurve_setControlPoint(lua_State *L)
 	if (idx > 0) // 1-indexing
 		idx--;
 
-	EXCEPT_GUARD(curve->setControlPoint(idx, Vector(vx,vy));)
+	luax_catchexcept(L, [&](){ curve->setControlPoint(idx, Vector(vx,vy)); });
 	return 0;
 }
 
@@ -89,7 +90,7 @@ int w_BezierCurve_insertControlPoint(lua_State *L)
 	if (idx > 0) // 1-indexing
 		idx--;
 
-	EXCEPT_GUARD(curve->insertControlPoint(Vector(vx,vy), idx);)
+	luax_catchexcept(L, [&](){ curve->insertControlPoint(Vector(vx,vy), idx); });
 	return 0;
 }
 
@@ -134,11 +135,11 @@ int w_BezierCurve_evaluate(lua_State *L)
 	BezierCurve *curve = luax_checkbeziercurve(L, 1);
 	double t = luaL_checknumber(L, 2);
 
-	EXCEPT_GUARD(
+	luax_catchexcept(L, [&]() {
 		Vector v = curve->evaluate(t);
 		lua_pushnumber(L, v.x);
 		lua_pushnumber(L, v.y);
-	)
+	});
 
 	return 2;
 
@@ -150,7 +151,7 @@ int w_BezierCurve_render(lua_State *L)
 	int accuracy = luaL_optinteger(L, 2, 5);
 
 	std::vector<Vector> points;
-	EXCEPT_GUARD(points = curve->render(accuracy);)
+	luax_catchexcept(L, [&](){ points = curve->render(accuracy); });
 
 	lua_createtable(L, points.size()*2, 0);
 	for (size_t i = 0; i < points.size(); ++i)
