@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -36,33 +36,23 @@ namespace sdl
 // we want them in pixel coordinates (may be different with high-DPI enabled.)
 static void windowToPixelCoords(int *x, int *y)
 {
-	double scale = 1.0;
+	window::Window *window = Module::getInstance<window::Window>(Module::M_WINDOW);
 
-	love::window::Window *window = love::window::sdl::Window::getSingleton();
-	if (window != nullptr)
-		scale = window->getPixelScale();
-
-	if (x != nullptr)
-		*x = int(double(*x) * scale);
-
-	if (y != nullptr)
-		*y = int(double(*y) * scale);
+	if (window && x)
+		*x = (int) window->toPixels(*x);
+	if (window && y)
+		*y = (int) window->toPixels(*y);
 }
 
 // And vice versa for setting mouse coordinates.
 static void pixelToWindowCoords(int *x, int *y)
 {
-	double scale = 1.0;
+	window::Window *window = Module::getInstance<window::Window>(Module::M_WINDOW);
 
-	love::window::Window *window = love::window::sdl::Window::getSingleton();
-	if (window != nullptr)
-		scale = window->getPixelScale();
-
-	if (x != nullptr)
-		*x = int(double(*x) / scale);
-
-	if (y != nullptr)
-		*y = int(double(*y) / scale);
+	if (window && x)
+		*x = (int) window->fromPixels(*x);
+	if (window && y)
+		*y = (int) window->fromPixels(*y);
 }
 
 const char *Mouse::getName() const
@@ -71,17 +61,17 @@ const char *Mouse::getName() const
 }
 
 Mouse::Mouse()
-	: curCursor(0)
+	: curCursor(nullptr)
 {
 }
 
 Mouse::~Mouse()
 {
-	if (curCursor)
+	if (curCursor.get())
 		setCursor();
 
-	for (auto it = systemCursors.begin(); it != systemCursors.end(); ++it)
-		it->second->release();
+	for (auto &c : systemCursors)
+		c.second->release();
 }
 
 love::mouse::Cursor *Mouse::newCursor(love::image::ImageData *data, int hotx, int hoty)
@@ -91,7 +81,7 @@ love::mouse::Cursor *Mouse::newCursor(love::image::ImageData *data, int hotx, in
 
 love::mouse::Cursor *Mouse::getSystemCursor(Cursor::SystemCursor cursortype)
 {
-	Cursor *cursor = NULL;
+	Cursor *cursor = nullptr;
 	auto it = systemCursors.find(cursortype);
 
 	if (it != systemCursors.end())
@@ -107,25 +97,19 @@ love::mouse::Cursor *Mouse::getSystemCursor(Cursor::SystemCursor cursortype)
 
 void Mouse::setCursor(love::mouse::Cursor *cursor)
 {
-	Object::AutoRelease cursorrelease(curCursor);
-
-	curCursor = cursor;
-	curCursor->retain();
-
+	curCursor.set(cursor);
 	SDL_SetCursor((SDL_Cursor *) cursor->getHandle());
 }
 
 void Mouse::setCursor()
 {
-	Object::AutoRelease cursorrelease(curCursor);
-	curCursor = NULL;
-
+	curCursor.set(nullptr);
 	SDL_SetCursor(SDL_GetDefaultCursor());
 }
 
 love::mouse::Cursor *Mouse::getCursor() const
 {
-	return curCursor;
+	return curCursor.get();
 }
 
 int Mouse::getX() const
@@ -166,6 +150,11 @@ void Mouse::setPosition(int x, int y)
 
 	pixelToWindowCoords(&x, &y);
 	SDL_WarpMouseInWindow(handle, x, y);
+
+	// SDL_WarpMouse doesn't directly update SDL's internal mouse state in Linux
+	// and Windows, so we call SDL_PumpEvents now to make sure the next
+	// getPosition call always returns the updated state.
+	SDL_PumpEvents();
 }
 
 void Mouse::setX(int x)
@@ -187,7 +176,7 @@ void Mouse::setVisible(bool visible)
 
 bool Mouse::isDown(Button *buttonlist) const
 {
-	Uint32 buttonstate = SDL_GetMouseState(0, 0);
+	Uint32 buttonstate = SDL_GetMouseState(nullptr, nullptr);
 
 	for (Button button = *buttonlist; button != BUTTON_MAX_ENUM; button = *(++buttonlist))
 	{
@@ -219,6 +208,16 @@ bool Mouse::isGrabbed() const
 		return false;
 }
 
+bool Mouse::setRelativeMode(bool relative)
+{
+	return SDL_SetRelativeMouseMode(relative ? SDL_TRUE : SDL_FALSE) == 0;
+}
+
+bool Mouse::getRelativeMode() const
+{
+	return SDL_GetRelativeMouseMode() != SDL_FALSE;
+}
+
 } // sdl
 } // mouse
 } // love
