@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -30,11 +30,11 @@ namespace love
 namespace mouse
 {
 
-static Mouse *instance = 0;
+#define instance() (Module::getInstance<Mouse>(Module::M_MOUSE))
 
 int w_newCursor(lua_State *L)
 {
-	Cursor *cursor = 0;
+	Cursor *cursor = nullptr;
 
 	if (lua_isstring(L, 1) || luax_istype(L, 1, FILESYSTEM_FILE_T) || luax_istype(L, 1, FILESYSTEM_FILE_DATA_T))
 		luax_convobj(L, 1, "image", "newImageData");
@@ -43,9 +43,10 @@ int w_newCursor(lua_State *L)
 	int hotx = luaL_optint(L, 2, 0);
 	int hoty = luaL_optint(L, 3, 0);
 
-	EXCEPT_GUARD(cursor = instance->newCursor(data, hotx, hoty);)
+	luax_catchexcept(L, [&](){ cursor = instance()->newCursor(data, hotx, hoty); });
 
 	luax_pushtype(L, "Cursor", MOUSE_CURSOR_T, cursor);
+	cursor->release();
 	return 1;
 }
 
@@ -58,9 +59,8 @@ int w_getSystemCursor(lua_State *L)
 		return luaL_error(L, "Invalid system cursor type: %s", str);
 
 	Cursor *cursor = 0;
-	EXCEPT_GUARD(cursor = instance->getSystemCursor(systemCursor);)
+	luax_catchexcept(L, [&](){ cursor = instance()->getSystemCursor(systemCursor); });
 
-	cursor->retain();
 	luax_pushtype(L, "Cursor", MOUSE_CURSOR_T, cursor);
 	return 1;
 }
@@ -70,24 +70,21 @@ int w_setCursor(lua_State *L)
 	// Revert to the default system cursor if no argument is given.
 	if (lua_isnoneornil(L, 1))
 	{
-		instance->setCursor();
+		instance()->setCursor();
 		return 0;
 	}
 
 	Cursor *cursor = luax_checkcursor(L, 1);
-	instance->setCursor(cursor);
+	instance()->setCursor(cursor);
 	return 0;
 }
 
 int w_getCursor(lua_State *L)
 {
-	Cursor *cursor = instance->getCursor();
+	Cursor *cursor = instance()->getCursor();
 
 	if (cursor)
-	{
-		cursor->retain();
 		luax_pushtype(L, "Cursor", MOUSE_CURSOR_T, cursor);
-	}
 	else
 		lua_pushnil(L);
 
@@ -96,20 +93,20 @@ int w_getCursor(lua_State *L)
 
 int w_getX(lua_State *L)
 {
-	lua_pushnumber(L, instance->getX());
+	lua_pushnumber(L, instance()->getX());
 	return 1;
 }
 
 int w_getY(lua_State *L)
 {
-	lua_pushnumber(L, instance->getY());
+	lua_pushnumber(L, instance()->getY());
 	return 1;
 }
 
 int w_getPosition(lua_State *L)
 {
 	int x, y;
-	instance->getPosition(x, y);
+	instance()->getPosition(x, y);
 	lua_pushinteger(L, x);
 	lua_pushinteger(L, y);
 	return 2;
@@ -118,14 +115,14 @@ int w_getPosition(lua_State *L)
 int w_setX(lua_State *L)
 {
 	int x = luaL_checkint(L, 1);
-	instance->setX(x);
+	instance()->setX(x);
 	return 0;
 }
 
 int w_setY(lua_State *L)
 {
 	int y = luaL_checkint(L, 1);
-	instance->setY(y);
+	instance()->setY(y);
 	return 0;
 }
 
@@ -133,7 +130,7 @@ int w_setPosition(lua_State *L)
 {
 	int x = luaL_checkint(L, 1);
 	int y = luaL_checkint(L, 2);
-	instance->setPosition(x, y);
+	instance()->setPosition(x, y);
 	return 0;
 }
 
@@ -151,7 +148,7 @@ int w_isDown(lua_State *L)
 	}
 	buttonlist[counter] = Mouse::BUTTON_MAX_ENUM;
 
-	luax_pushboolean(L, instance->isDown(buttonlist));
+	luax_pushboolean(L, instance()->isDown(buttonlist));
 	delete[] buttonlist;
 	return 1;
 }
@@ -159,26 +156,39 @@ int w_isDown(lua_State *L)
 int w_setVisible(lua_State *L)
 {
 	bool b = luax_toboolean(L, 1);
-	instance->setVisible(b);
+	instance()->setVisible(b);
 	return 0;
 }
 
 int w_isVisible(lua_State *L)
 {
-	luax_pushboolean(L, instance->isVisible());
+	luax_pushboolean(L, instance()->isVisible());
 	return 1;
 }
 
 int w_setGrabbed(lua_State *L)
 {
 	bool b = luax_toboolean(L, 1);
-	instance->setGrabbed(b);
+	instance()->setGrabbed(b);
 	return 0;
 }
 
 int w_isGrabbed(lua_State *L)
 {
-	luax_pushboolean(L, instance->isGrabbed());
+	luax_pushboolean(L, instance()->isGrabbed());
+	return 1;
+}
+
+int w_setRelativeMode(lua_State *L)
+{
+	bool relative = luax_toboolean(L, 1);
+	luax_pushboolean(L, instance()->setRelativeMode(relative));
+	return 1;
+}
+
+int w_getRelativeMode(lua_State *L)
+{
+	luax_pushboolean(L, instance()->getRelativeMode());
 	return 1;
 }
 
@@ -200,6 +210,8 @@ static const luaL_Reg functions[] =
 	{ "getPosition", w_getPosition },
 	{ "setGrabbed", w_setGrabbed },
 	{ "isGrabbed", w_isGrabbed },
+	{ "setRelativeMode", w_setRelativeMode },
+	{ "getRelativeMode", w_getRelativeMode },
 	{ 0, 0 }
 };
 
@@ -212,9 +224,10 @@ static const lua_CFunction types[] =
 
 extern "C" int luaopen_love_mouse(lua_State *L)
 {
-	if (instance == 0)
+	Mouse *instance = instance();
+	if (instance == nullptr)
 	{
-		EXCEPT_GUARD(instance = new love::mouse::sdl::Mouse();)
+		luax_catchexcept(L, [&](){ instance = new love::mouse::sdl::Mouse(); });
 	}
 	else
 		instance->retain();
