@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -142,6 +142,15 @@ void Contact::getChildren(int &childA, int &childB)
 	childB = contact->GetChildIndexB();
 }
 
+void Contact::getFixtures(Fixture *&fixtureA, Fixture *&fixtureB)
+{
+	fixtureA = (Fixture *) Memoizer::find(contact->GetFixtureA());
+	fixtureB = (Fixture *) Memoizer::find(contact->GetFixtureB());
+
+	if (!fixtureA || !fixtureB)
+		throw love::Exception("A fixture has escaped Memoizer!");
+}
+
 } // box2d
 } // physics
 } // love
