@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -39,6 +39,16 @@ namespace box2d
 class Body;
 class World;
 
+/**
+ * This struct is stored in a void pointer in the Box2D Joint class. For now, all
+ * we need is a Lua reference to arbitrary data, but we might need more later.
+ **/
+struct jointudata
+{
+    // Reference to arbitrary data.
+    Reference *ref;
+};
+
 /**
  * A Joint acts as positioning constraints on Bodies.
  * A Joint can be used to prevent Bodies from going to
@@ -69,12 +79,14 @@ public:
 	 **/
 	bool isValid() const;
 
-
 	/**
 	 * Gets the type of joint.
 	 **/
 	Type getType() const;
 
+	Body *getBodyA() const;
+	Body *getBodyB() const;
+
 	/**
 	 * Gets the anchor positions of the Joint in world
 	 * coordinates. This is useful for debugdrawing the joint.
@@ -95,6 +107,17 @@ public:
 
 	bool getCollideConnected() const;
 
+	/**
+	 * This function stores an in-C reference to arbitrary Lua data in the Box2D
+	 * Joint object.
+	 **/
+	int setUserData(lua_State *L);
+
+	/**
+	 * Gets the data set with setUserData. If no data is set, nil is returned.
+	 **/
+	int getUserData(lua_State *L);
+
 	/**
 	 * Joints require pointers to a Box2D joint objects at
 	 * different polymorphic levels, which is why these function
@@ -117,6 +140,8 @@ protected:
 
 	World *world;
 
+    jointudata *udata;
+
 private:
 
 	// A Joint must be destroyed *before* the bodies it acts upon,
