@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -241,7 +241,7 @@ int w_Body_setX(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setX(arg1);)
+	luax_catchexcept(L, [&](){ t->setX(arg1); });
 	return 0;
 }
 
@@ -249,7 +249,7 @@ int w_Body_setY(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setY(arg1);)
+	luax_catchexcept(L, [&](){ t->setY(arg1); });
 	return 0;
 }
 
@@ -266,7 +266,7 @@ int w_Body_setAngle(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setAngle(arg1);)
+	luax_catchexcept(L, [&](){ t->setAngle(arg1); });
 	return 0;
 }
 
@@ -283,14 +283,14 @@ int w_Body_setPosition(lua_State *L)
 	Body *t = luax_checkbody(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
 	float arg2 = (float)luaL_checknumber(L, 3);
-	EXCEPT_GUARD(t->setPosition(arg1, arg2);)
+	luax_catchexcept(L, [&](){ t->setPosition(arg1, arg2); });
 	return 0;
 }
 
 int w_Body_resetMassData(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
-	EXCEPT_GUARD(t->resetMassData();)
+	luax_catchexcept(L, [&](){ t->resetMassData(); });
 	return 0;
 }
 
@@ -301,7 +301,7 @@ int w_Body_setMassData(lua_State *L)
 	float y = (float)luaL_checknumber(L, 3);
 	float m = (float)luaL_checknumber(L, 4);
 	float i = (float)luaL_checknumber(L, 5);
-	EXCEPT_GUARD(t->setMassData(x, y, m, i);)
+	luax_catchexcept(L, [&](){ t->setMassData(x, y, m, i); });
 	return 0;
 }
 
@@ -309,7 +309,7 @@ int w_Body_setMass(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	float m = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setMass(m);)
+	luax_catchexcept(L, [&](){ t->setMass(m); });
 	return 0;
 }
 
@@ -317,7 +317,7 @@ int w_Body_setInertia(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	float i = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setInertia(i);)
+	luax_catchexcept(L, [&](){ t->setInertia(i); });
 	return 0;
 }
 
@@ -351,7 +351,7 @@ int w_Body_setType(lua_State *L)
 	const char *typeStr = luaL_checkstring(L, 2);
 	Body::Type type;
 	Body::getConstant(typeStr, type);
-	EXCEPT_GUARD(t->setType(type);)
+	luax_catchexcept(L, [&](){ t->setType(type); });
 	return 0;
 }
 
@@ -494,7 +494,7 @@ int w_Body_setActive(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	bool b = luax_toboolean(L, 2);
-	EXCEPT_GUARD(t->setActive(b);)
+	luax_catchexcept(L, [&](){ t->setActive(b); });
 	return 0;
 }
 
@@ -510,7 +510,7 @@ int w_Body_setFixedRotation(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	bool b = luax_toboolean(L, 2);
-	EXCEPT_GUARD(t->setFixedRotation(b);)
+	luax_catchexcept(L, [&](){ t->setFixedRotation(b); });
 	return 0;
 }
 
@@ -522,22 +522,55 @@ int w_Body_isFixedRotation(lua_State *L)
 	return 1;
 }
 
+int w_Body_getWorld(lua_State *L)
+{
+	Body *t = luax_checkbody(L, 1);
+	World *world = t->getWorld();
+	luax_pushtype(L, "World", PHYSICS_WORLD_T, world);
+	return 1;
+}
+
 int w_Body_getFixtureList(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
 	lua_remove(L, 1);
 	int n = 0;
-	EXCEPT_GUARD(n = t->getFixtureList(L);)
+	luax_catchexcept(L, [&](){ n = t->getFixtureList(L); });
+	return n;
+}
+
+int w_Body_getJointList(lua_State *L)
+{
+	Body *t = luax_checkbody(L, 1);
+	lua_remove(L, 1);
+	int n = 0;
+	luax_catchexcept(L, [&](){ n = t->getJointList(L); });
+	return n;
+}
+
+int w_Body_getContactList(lua_State *L)
+{
+	Body *t = luax_checkbody(L, 1);
+	lua_remove(L, 1);
+	int n = 0;
+	luax_catchexcept(L, [&](){ n = t->getContactList(L); });
 	return n;
 }
 
 int w_Body_destroy(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
-	EXCEPT_GUARD(t->destroy();)
+	luax_catchexcept(L, [&](){ t->destroy(); });
 	return 0;
 }
 
+int w_Body_isDestroyed(lua_State *L)
+{
+	Body *b = luax_checktype<Body>(L, 1, "Body", PHYSICS_BODY_T);
+	luax_pushboolean(L, b->body == nullptr);
+	return 1;
+}
+
 int w_Body_setUserData(lua_State *L)
 {
 	Body *t = luax_checkbody(L, 1);
@@ -604,8 +637,12 @@ static const luaL_Reg functions[] =
 	{ "setAwake", w_Body_setAwake },
 	{ "setFixedRotation", w_Body_setFixedRotation },
 	{ "isFixedRotation", w_Body_isFixedRotation },
+	{ "getWorld", w_Body_getWorld },
 	{ "getFixtureList", w_Body_getFixtureList },
+	{ "getJointList", w_Body_getJointList },
+	{ "getContactList", w_Body_getContactList },
 	{ "destroy", w_Body_destroy },
+	{ "isDestroyed", w_Body_isDestroyed },
 	{ "setUserData", w_Body_setUserData },
 	{ "getUserData", w_Body_getUserData },
 	{ 0, 0 }
