@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -42,11 +42,11 @@ class Sound : public Module
 
 public:
 
-	/**
-	 * Destructor.
-	 **/
 	virtual ~Sound();
 
+	// Implements Module.
+	virtual ModuleType getModuleType() const { return M_SOUND; }
+
 	/**
 	 * Creates new SoundData from a decoder. Fully expands the
 	 * encoded sound data into raw sound data. Not recommended
