@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -37,7 +37,6 @@ Decoder::Decoder(Data *data, const std::string &ext, int bufferSize)
 	, buffer(0)
 	, eof(false)
 {
-	data->retain();
 	buffer = new char[bufferSize];
 }
 
@@ -45,8 +44,6 @@ Decoder::~Decoder()
 {
 	if (buffer != 0)
 		delete [](char *) buffer;
-	if (data != 0)
-		data->release();
 }
 
 void *Decoder::getBuffer() const
