@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2015 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -33,11 +33,11 @@ class Timer : public Module
 {
 public:
 
-	/**
-	 * Destructor.
-	 **/
 	virtual ~Timer() {}
 
+	// Implements Module.
+	virtual ModuleType getModuleType() const { return M_TIMER; }
+
 	/**
 	 * Measures the time between this call and the previous call,
 	 * and updates internal values accordinly.
