@@ -7,24 +7,24 @@
 local max_width = 18
 local pattern = [[
 /**
-* Copyright (c) 2006-2012 LOVE Development Team
-* 
-* This software is provided 'as-is', without any express or implied
-* warranty.  In no event will the authors be held liable for any damages
-* arising from the use of this software.
-* 
-* Permission is granted to anyone to use this software for any purpose,
-* including commercial applications, and to alter it and redistribute it
-* freely, subject to the following restrictions:
-* 
-* 1. The origin of this software must not be misrepresented; you must not
-*    claim that you wrote the original software. If you use this software
-*    in a product, an acknowledgment in the product documentation would be
-*    appreciated but is not required.
-* 2. Altered source versions must be plainly marked as such, and must not be
-*    misrepresented as being the original software.
-* 3. This notice may not be removed or altered from any source distribution.
-**/
+ * Copyright (c) 2006-2015 LOVE Development Team
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.  In no event will the authors be held liable for any damages
+ * arising from the use of this software.
+ * 
+ * Permission is granted to anyone to use this software for any purpose,
+ * including commercial applications, and to alter it and redistribute it
+ * freely, subject to the following restrictions:
+ * 
+ * 1. The origin of this software must not be misrepresented; you must not
+ *    claim that you wrote the original software. If you use this software
+ *    in a product, an acknowledgment in the product documentation would be
+ *    appreciated but is not required.
+ * 2. Altered source versions must be plainly marked as such, and must not be
+ *    misrepresented as being the original software.
+ * 3. This notice may not be removed or altered from any source distribution.
+ **/
 
 namespace love
 {
@@ -34,7 +34,8 @@ const unsigned char %s[] =
 {
 %s
 }; // [%s]
-} // love]]
+} // love
+]]
 --formatting parameters:
 -- - input file name
 -- - c variable name
@@ -107,7 +108,8 @@ end
 for i, v in ipairs(arg) do
 	--run the auto function for every argument
 	--but do it with pcall, to catch errors
-	local ok, err = pcall(auto, v:gsub("%.lua$",""))
+	v = v:gsub("%.lua$", ""):gsub("^(.+)/", "") -- normalize input
+	local ok, err = pcall(auto, v)
 	if not ok then
 		--inform people we've failed
 		print(v .. ": " .. err)
